import re
import subprocess

from termcolor import colored
from src.config import get_config
import click
import tempfile
from pathlib import Path

pathDataRe = re.compile(r"path\.data\s?=", re.IGNORECASE)


@click.command(help="Runs elastic search from the current kibana clone. It will use parameters from the ~/.kibbe [elastic.params] section.")
@click.option('--data-dir', '-d', type=click.STRING, default="esdata", help="Path where this elastic search will store its data (path.data)")
@click.option('--no-persist', '-n', default=False, is_flag=True, help="If passed will use a disposable data dir. This option will overwrite other options related to data dir.")
@click.option('-E', multiple=True, help="Additional options to pass to elastic search. `path.data` will be ignored")
def es(data_dir, no_persist, e):

    params = process_params(data_dir, no_persist)

    # additional -E params
    for item in e:
        item = item.strip()
        # ignore path.data
        if pathDataRe.match(item):
            continue
        params.append(item)

    command = get_command(params)
    click.echo("Will run elastic search as: " + colored(' '.join(command), 'yellow'))
    subprocess.run(command)


def get_command(params):
    final_params = []
    for param in params:
        final_params.append('-E')
        final_params.append(param)

    return ['node', 'scripts/es', 'snapshot'] + final_params


def process_params(data_dir, no_persist):
    CONFIG_KEY = 'elastic.params'
    config = get_config()
    params = []
    if CONFIG_KEY in config:
        for (key, value) in config.items(CONFIG_KEY, raw=True):
            # ignore path.data if this command overwrites it
            if key == 'path.data':
                if len(data_dir) > 0:
                    value = get_data_dir(data_dir, no_persist)
                else:
                    value = get_data_dir(value, no_persist)

            if len(value) > 0:
                params.append(str(key) + '=' + str(value))
            else:
                params.append(str(key))

    return params


def get_data_dir(data_dir, no_persist):
    if no_persist or len(data_dir) == 0:
        return tempfile.mkdtemp(suffix='kibbe')

    return str(Path(data_dir).resolve())



import json
import os
import subprocess
from pathlib import PurePath

import click


def is_tool(name):
    """Check whether `name` is on PATH and marked as executable."""

    # from whichcraft import which
    from shutil import which

    return which(name) is not None


def force_kibana_root():
    if not is_kibana_repo():
        raise click.ClickException("You must run this command in the root of a kibana repo clone")


def is_kibana_repo():
    if not os.path.isfile('package.json'):
        return False

    file = open('package.json')
    try:
        content = json.load(file)
        if content['name'] != 'kibana' or not content['homepage'].startswith('https://www.elastic.co/'):
            return False
    except ValueError:
        return False
    finally:
        file.close()

    return True


def get_modified_files():
    files = ""
    try:
        files = subprocess.getoutput('git diff --name-only HEAD')
    except ValueError:
        return []

    files = filter(None, files.split('\n'))
    return list(files)


def find_related_test(file):
    path = PurePath(file)

    # skip if the file is a test
    if path.match('*.test.*'):
        return ""

    test_file = path.with_suffix('.test' + path.suffix)

    if os.path.isfile(test_file):
        return test_file

    return ""


def find_related_plugin_folder(file):
    path = PurePath(file)

    try:
        if not path.relative_to('x-pack/plugins'):
            return ""
    except ValueError:
        return ""

    while not path.match('x-pack/plugins/*'):
        path = PurePath(path.parent)

    return str(path)



import os
import subprocess
from pathlib import PurePath

import click
from termcolor import colored
from src.util import (find_related_plugin_folder, find_related_test,
                      force_kibana_root, get_modified_files)


@click.command(help="""Runs quick checks for your modified files. Useful to run before committing""")
@click.option('--skip-tests', default=False, help="Skip running tests", is_flag=True)
@click.option('--skip-stylescheck', default=False, help="Skip running styles check", is_flag=True)
@click.option('--check-types', default=False, help="Runs Typescript types check (if relevant)", is_flag=True)
def check(skip_tests, check_types, skip_stylescheck):
    force_kibana_root()

    modified = get_modified_files()

    if len(modified) == 0:
        click.echo(colored("Kibbe check only works with uncommited files. Edit some files first", "yellow"))

    configs = set()
    tests = set()
    styles = set()
    for file in modified:
        path = PurePath(file)
        if path.suffix in ['.css', '.scss']:
            styles.add(file)
        tests.add(find_related_test(file))
        configs.add(find_related_plugin_folder(file))

    tests = list(filter(None, tests))
    configs = list(filter(None, configs))
    styles = list(filter(None, styles))

    fails = []

    # style check
    if not skip_stylescheck and len(styles) > 0:
        click.echo(colored('>> Running styles check', 'yellow'))
        process = subprocess.run(['node', 'scripts/stylelint.js'] + styles)
        if process.returncode != 0:
            fails.append("Stylelint")

    # types check
    if check_types and len(configs) > 0:
        click.echo(colored('>> Finding tsconfig.json', 'yellow'))
        for config in configs:
            tsconfig = os.path.join(config, 'tsconfig.json')
            if os.path.isfile(tsconfig):
                click.echo(colored('>>  - Running type_check for ', 'cyan') + colored(tsconfig, 'white'))
                process = subprocess.run(['node', 'scripts/type_check.js', '--project', tsconfig])
                if process.returncode != 0:
                    fails.append("TS types check")

    # eslint
    if len(modified) > 0:
        click.echo(colored('>> Running eslint', 'yellow'))
        process = subprocess.run(['node', 'scripts/eslint.js'] + modified)
        if process.returncode != 0:
            fails.append("Eslint checks")

    # tests
    if not skip_tests and len(tests) > 0:
        click.echo(colored('>> Running related tests', 'yellow'))
        process = subprocess.run(['node', 'scripts/jest.js'] + tests)
        if process.returncode != 0:
            fails.append("Jest tests")

    if len(fails) > 0:
        click.echo("\n ------ \n")
        click.echo(colored(">> Errors found running checks:\n", 'red'))
        for fail in fails:
            click.echo('      ❌ - ' + str(fail))



import subprocess
from setuptools import setup, find_packages

# takes the version from the latest current tag
version = subprocess.getoutput('git describe --tags --abbrev=0')[1:]

setup(
    name='kibbe',
    version='0.10.1',
    packages=find_packages(),
    include_package_data=True,
    download_url='https://github.com/academo/kibbe/releases',
    url='https://github.com/academo/kibbe/',
    author="Esteban Beltran",
    author_email="kibbe@academo.me",
    install_requires=['click', 'termcolor'],
    entry_points={
        'console_scripts': [
            'kibbe = src.main:cli',
        ],
    },
)



from pathlib import Path, PurePath
import configparser
from termcolor import colored

import click


def get_config():
    config = configparser.ConfigParser()
    try:
        config_file_path = PurePath(Path.home()).joinpath('.kibbe')
        config.read(config_file_path)
    except ValueError:
        click.echo(colored('Error reading your configuration file.', 'red'))
        pass
    return config
